/*
 * @(#)phase.c	1.4 01/30/01
 *
 * Copyright (C) 1997-2001 Tak-Shing Chan.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 */

#include <Xm/Xm.h>
#include <Xm/Form.h>
#include <Xm/Label.h>
#include <Xm/TextF.h>
#include <Xm/Scale.h>
#include <Xm/Frame.h>
#include <Xm/RowColumn.h>
#include <Xm/ToggleBG.h>
#include <Xm/List.h>
#include <Xm/DrawingA.h>
#include <Xm/FileSB.h>
#include <Xm/MessageB.h>
#include <Xm/SelectioB.h>
#include <Xm/Text.h>
#include <errno.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* Panel settings and associated widgets */
int         parts = 2, bpm = 120, scale = 0, length = 2, ht = 1,
            rs = 1, duration = 100, func[100];
Widget      app_shell, Seed, Parts, BPM, Scale, Length, HT, RS,
            Duration;

/* Pixmap for phase function */
Display     *display;
Drawable    d;
GC          gc;
Pixmap      pixmap;
int         x,  y;

/* Error handler */
/* Error: True = I/O error, False = format error */
void
alert(char *fname, int error)
{
    char        foo[256];
    XmString    msg;
    Widget      ad;

    if (error)
        sprintf(foo, "Error Alert! \nUnable to access %s. \n\
%.64s. ", fname, strerror(errno));
    else
        sprintf(foo, "Error Alert! \nUnable to read %s. \n\
Invalid file format. ", fname);

    msg = XmStringCreateLtoR(foo, XmFONTLIST_DEFAULT_TAG);
    ad = XmCreateErrorDialog(app_shell, "alert", NULL, 0);
    XtVaSetValues(ad, XmNmessageString, msg, NULL);
    XtUnmanageChild(XmMessageBoxGetChild(ad, XmDIALOG_CANCEL_BUTTON));
    XtUnmanageChild(XmMessageBoxGetChild(ad, XmDIALOG_HELP_BUTTON));
    XtManageChild(ad);
    XmStringFree(msg);
}

/* Show phase function */
void
showphase(void)
{
    XGCValues   values;
    int         i;

    XFillRectangle(display, pixmap, gc, 0, 0, 200, 50);
    values.function = GXxor;
    XChangeGC(display, gc, GCFunction, &values);
    for (i = 0; i < 100; i++)
        XDrawLine(display, pixmap, gc, (i << 1) + 1, 0,
            (i << 1) + 1, func[i]);
    values.function = GXcopy;
    XChangeGC(display, gc, GCFunction, &values);
    XCopyArea(display, pixmap, d, gc, 0, 0, 200, 50, 0, 0);
}

/* Reset phase function */
void
resetphase(void)
{
    int i;

    for (i = 0; i < 100; i++)
        func[i] = 40;
    showphase();
}

/* Draw a pixel */
void
drawpixel(int x, int y)
{
    if (y < 0)
        y = 0;
    if (y > 49)
        y = 49;
    if (x >= 0 && x < 100)
        func[x] = y;
}

/* Callback for Cancel */
void
CancelCB(Widget w, XtPointer client_data, XtPointer call_data)
{
    XtUnmanageChild(w);
}

/* Callback for Open OK and Save As OK */
/* Client: True = Open OK, False = Save As OK */
void
OpenCB(Widget w, XtPointer client_data, XtPointer call_data)
{
    char    *fname, foo[32], *bar;
    FILE    *fp;
    int     seed, i;
    Widget  button;

    fname = XmTextGetString(XmFileSelectionBoxGetChild(w, XmDIALOG_TEXT));

    if ((int) client_data) {
        if (!(fp = fopen(fname, "rb")))
            alert(fname, True);
        else {
            if (!fgets(foo, sizeof(foo), fp)
                || strcmp(foo, "Phase Generator Data\n"))
                alert(fname, False);
            else {
                sprintf(foo, "button_%d", parts - 2);
                button = XtNameToWidget(Parts, foo);
                XmToggleButtonGadgetSetState(button, False, False);
                fscanf(fp, "%d%d%d%d%d%d%d%d", &seed, &parts,
                    &bpm, &scale, &length, &ht, &rs, &duration);
                sprintf(foo, "%d", seed);
                XmTextFieldSetString(Seed, foo);
                sprintf(foo, "button_%d", parts - 2);
                button = XtNameToWidget(Parts, foo);
                XmToggleButtonGadgetSetState(button, True, False);
                XmScaleSetValue(BPM, bpm);
                XmListSelectPos(Scale, scale + 1, False);
                XmScaleSetValue(Length, length);
                XmScaleSetValue(HT, ht);
                XmScaleSetValue(RS, rs);
                XmScaleSetValue(Duration, duration);
                for (i = 0; i < 100; i++)
                    fscanf(fp, "%d", &func[i]);
                showphase();
            }
            fclose(fp);
        }
    }
    else {
        bar = XmTextFieldGetString(Seed);
        if (!(fp = fopen(fname, "wb")))
            alert(fname, True);
        else {
            fprintf(fp,
                "Phase Generator Data\n%d %d %d %d %d %d %d %d\n",
                atoi(bar), parts, bpm, scale, length, ht, rs, duration);
                for (i = 0; i < 100; i++)
                    fprintf(fp, "%d\n", func[i]);
            fclose(fp);
        }
        XtFree(bar);
    }
    XtFree(fname);
    XtUnmanageChild(w);
}

/* Callback for File menu */
/* Client: 0 = Open, 1 = Save As, 2 = Exit */
void
FileCB(Widget w, XtPointer client_data, XtPointer call_data)
{
    static Widget   od = NULL, sd = NULL;

    switch ((int) client_data) {
    case 0:
        if (!od) {
            od = XmCreateFileSelectionDialog(app_shell, "open", NULL, 0);
            XtVaSetValues(od, XmNdialogStyle, XmDIALOG_SYSTEM_MODAL, NULL);
            XtAddCallback(od, XmNokCallback, OpenCB, (XtPointer) True);
            XtAddCallback(od, XmNcancelCallback, CancelCB, NULL);
            XtUnmanageChild(XmFileSelectionBoxGetChild(od,
                XmDIALOG_HELP_BUTTON));
        }
        XtManageChild(od);
        break;
    case 1:
        if (!sd) {
            sd = XmCreateFileSelectionDialog(app_shell, "save", NULL, 0);
            XtVaSetValues(sd, XmNdialogStyle, XmDIALOG_SYSTEM_MODAL, NULL);
            XtAddCallback(sd, XmNokCallback, OpenCB, (XtPointer) False);
            XtAddCallback(sd, XmNcancelCallback, CancelCB, NULL);
            XtUnmanageChild(XmFileSelectionBoxGetChild(sd,
                XmDIALOG_HELP_BUTTON));
        }
        XtManageChild(sd);
        break;
    case 2:
        exit(EXIT_SUCCESS);
    }
}

/* Callback for Make MIDI File OK and Play Music OK */
/* Client: True = Make MIDI File OK, False = Play Music OK */
void
MakeCB(Widget w, XtPointer client_data, XtPointer call_data)
{
    unsigned char   scales[][8] = { { 60, 62, 64, 67, 69, 72, 74, 76 },
                                    { 60, 62, 64, 65, 67, 69, 71, 72 },
                                    { 60, 62, 63, 65, 67, 68, 70, 72 },
                                    { 60, 62, 63, 65, 67, 69, 71, 72 },
                                    { 60, 62, 63, 65, 67, 68, 71, 72 },
                                    { 60, 62, 64, 66, 68, 70, 72, 74 },
                                    { 60, 62, 63, 65, 66, 68, 69, 71 },
                                    { 60, 61, 62, 63, 64, 65, 66, 67 } };
    unsigned char   midi[] = { 'M', 'T', 'h', 'd', 0, 0, 0, 6,
                               0, 1, 0, 0, 0, 192,
                               'M', 'T', 'r', 'k', 0, 0, 0, 19,
                               0, 0xff, 0x58, 4, 4, 2, 24, 8,
                               0, 0xff, 0x51, 3, 0, 0, 0,
                               0, 0xff, 0x2f, 0 };
    unsigned char   track[10000] = { 'M', 'T', 'r', 'k' };
    char            *fname, *seed, tmp[L_tmpnam], table[24], *mp, *cmd;
    FILE            *fp;
    int             make = (int) client_data, foo, total, note, dur, i, j;
    long            bar, pos;

    fname = make ? XmTextGetString(XmFileSelectionBoxGetChild(w,
        XmDIALOG_TEXT)) : tmpnam(tmp);

    if (!(fp = fopen(fname, "wb")))
        alert(fname, True);
    else {
        /* Header chuck */
        bar = 60000000 / bpm;                   /* Tempo */
        midi[11] = parts + 1;
        midi[34] = bar >> 16;
        midi[35] = (bar >> 8) & 0xff;
        midi[36] = bar & 0xff;
        fwrite(midi, 1, 41, fp);

        /* Track chucks */
#define midiout track[pos++] =
        seed = XmTextFieldGetString(Seed);
        srand((unsigned int) atoi(seed));
        XtFree(seed);
        for (i = 0, foo = rand() % 12 - 6; i < length; i++)
            table[i] = foo + scales[scale][rand() % 8];
        total = 96 * duration;
        for (i = 0; i < parts; i++) {
            pos = 8;
            midiout 0;
            midiout 0xc0 + i;
            midiout rand() % 16;
            midiout 0;
            midiout 0x90 + i;                   /* Running status */
            for (j = 0, foo = total; foo > 48; j++) {
                note = table[j % length];
                midiout note;
                midiout 64 - (ht >> 1) + rand() % ht;
                dur = 96 - i * ((49 - func[99 - 99 * foo / total]) >> 3);
                bar = rand() % rs;
                midiout dur - bar;
                midiout note;
                midiout 0;
                midiout bar;
                foo -= dur;
            }
            note = table[j % length];
            midiout note;                       /* Ending note */
            midiout 64 - (ht >> 1) + rand() % ht;
            midiout (768 >> 7) | 0x80;
            midiout 768 & 0x7f;
            midiout note;
            midiout 0;
            midiout 0;
            midiout 0xff;
            midiout 0x2f;
            midiout 0;
            bar = pos - 8;                      /* Track length */
            track[4] = bar >> 24;
            track[5] = (bar >> 16) & 0xff;
            track[6] = (bar >> 8) & 0xff;
            track[7] = bar & 0xff;
            fwrite(track, 1, pos, fp);
        }
        fclose(fp);

        if (!make) {
            mp = XmTextGetString(XmSelectionBoxGetChild(w, XmDIALOG_TEXT));
            cmd = XtMalloc(strlen(mp) + strlen(fname) + 2);
            sprintf(cmd, "%s %s", mp, fname);
            system(cmd);
            XtFree(cmd);
            XtFree(mp);
            remove(fname);
        }
    }
    if (make) {
        XtFree(fname);
        XtUnmanageChild(w);
    }
}

/* Callback for Music menu */
/* Client: 0 = Make MIDI File, 1 = Play Music */
void
MusicCB(Widget w, XtPointer client_data, XtPointer call_data)
{
    static Widget   md = NULL, pd = NULL;
    XmString        msg;

    switch ((int) client_data) {
    case 0:
        if (!md) {
            md = XmCreateFileSelectionDialog(app_shell, "make", NULL, 0);
            XtVaSetValues(md, XmNdialogStyle, XmDIALOG_SYSTEM_MODAL, NULL);
            XtAddCallback(md, XmNokCallback, MakeCB, (XtPointer) True);
            XtAddCallback(md, XmNcancelCallback, CancelCB, NULL);
            XtUnmanageChild(XmFileSelectionBoxGetChild(md,
                XmDIALOG_HELP_BUTTON));
        }
        XtManageChild(md);
        break;
    case 1:
        if (!pd) {
            msg = XmStringCreateSimple("Enter pathname of the MIDI player: ");
            pd = XmCreatePromptDialog(app_shell, "play", NULL, 0);
            XtVaSetValues(pd, XmNselectionLabelString, msg, NULL);
            XtAddCallback(pd, XmNokCallback, MakeCB, (XtPointer) False);
            XtAddCallback(pd, XmNcancelCallback, CancelCB, NULL);
            XtUnmanageChild(XmSelectionBoxGetChild(pd, XmDIALOG_HELP_BUTTON));
            XmStringFree(msg);
        }
        XtManageChild(pd);
    }
}

/* Callback for Help menu */
void
HelpCB(Widget w, XtPointer client_data, XtPointer call_data)
{
    static Widget   ad = NULL;
    XmString        msg;

    if (!ad) {
        msg = XmStringCreateLtoR("About Random Phase Generator 1.3: \n\
Copyright (C) 1997-2001 Tak-Shing Chan. \n\
Under GNU GPL Version 2 or later.\n\
Comments to chan12@engr.newpaltz.edu. ", XmFONTLIST_DEFAULT_TAG);
        ad = XmCreateInformationDialog(app_shell, "about", NULL, 0);
        XtVaSetValues(ad, XmNmessageString, msg, NULL);
        XtUnmanageChild(XmMessageBoxGetChild(ad, XmDIALOG_CANCEL_BUTTON));
        XtUnmanageChild(XmMessageBoxGetChild(ad, XmDIALOG_HELP_BUTTON));
        XmStringFree(msg);
    }
    XtManageChild(ad);
}

/* Callback for Number of Parts buttons */
void
PartsCB(Widget w, XtPointer client_data, XtPointer call_data)
{
    parts = (int) client_data + 2;
}

/* Callback for Beats Per Minute slider */
void
BPMCB(Widget w, XtPointer client_data, XtPointer call_data)
{
    bpm = ((XmScaleCallbackStruct *) call_data)->value;
}

/* Callback for Scale selector */
void
ScaleCB(Widget w, XtPointer client_data, XtPointer call_data)
{
    scale = ((XmListCallbackStruct *) call_data)->item_position - 1;
}

/* Callback for Phase Function expose */
void
ExposeCB(Widget w, XtPointer client_data, XtPointer call_data)
{
    XCopyArea(display, pixmap, d, gc, 0, 0, 200, 50, 0, 0);
}

/* Callback for Phase Function input */
void
InputCB(Widget w, XtPointer client_data, XtPointer call_data)
{
    XEvent  *event = ((XmDrawingAreaCallbackStruct *) call_data)->event;

    if (event && event->xany.type == ButtonPress) {
        switch (event->xbutton.button) {
        case Button1:
              x = event->xbutton.x >> 1;
              y = event->xbutton.y;
              drawpixel(x, y);
              showphase();
              break;
        case Button3:
              resetphase();
        }
    }
}

/* Callback for Phase Function motion */
void
MotionCB(Widget w, XtPointer client_data, XEvent *event, Boolean *next)
{
    int x0, x1, x2, y0, y1, y2, dx, dy, d1, d2, d, i;

    if (event->type == MotionNotify &&
        event->xmotion.state == Button1Mask) {
        x0 = event->xbutton.x >> 1;
        y0 = event->xbutton.y;

        /* Bresenham's midpoint line algorithm */
        x1 = x2 = (x0 > x) ? 1 : -1;
        y1 = y2 = (y0 > y) ? 1 : -1;
        dx = abs(x0 - x);
        dy = abs(y0 - y);
        if (dy > dx) {
            i = dy + 1;
            x1 = 0;
            d1 = dx << 1;
            d2 = d1 - (dy << 1);
            d = d1 - dy;
        } else {
            i = dx + 1;
            y1 = 0;
            d1 = dy << 1;
            d2 = d1 - (dx << 1);
            d = d1 - dx;
        }
        while (i--) {
            drawpixel(x, y);
            if (d < 0) {
                x += x1;
                y += y1;
                d += d1;
            } else {
                x += x2;
                y += y2;
                d += d2;
            }
        }
        x = x0;
        y = y0;
        showphase();
    }
}

/* Callback for Pattern Length slider */
void
LengthCB(Widget w, XtPointer client_data, XtPointer call_data)
{
    length = ((XmScaleCallbackStruct *) call_data)->value;
}

/* Callback for Humanize Touch slider */
void
HTCB(Widget w, XtPointer client_data, XtPointer call_data)
{
    ht = ((XmScaleCallbackStruct *) call_data)->value;
}

/* Callback for Random Staccato slider */
void
RSCB(Widget w, XtPointer client_data, XtPointer call_data)
{
    rs = ((XmScaleCallbackStruct *) call_data)->value;
}

/* Callback for Total Duration slider */
void
DurationCB(Widget w, XtPointer client_data, XtPointer call_data)
{
    duration = ((XmScaleCallbackStruct *) call_data)->value;
}

int
main(int argc, char **argv)
{
    XtAppContext    app;
    String          fallback_resources[] = {
        "phase.*.title.fontList: -*-helvetica-bold-r-normal-*-14-*-iso8859-1",
        "phase.*.fontList: -*-helvetica-medium-r-normal-*-14-*-iso8859-1",
        "phase.*.foreground: #fafafafad2d2",
        "phase.*.background: #2f2f4f4f4f4f",
        NULL
    };
    Widget          form, menu, frame1, frame2, draw;
    XmString        foo[9];
    Screen          *screen;
    XColor          screen_def, exact_def;
    XGCValues       values;

    app_shell = XtVaAppInitialize(&app, "phase", NULL, 0, &argc, argv,
        fallback_resources,
        XmNwidth, 640,
        XmNheight, 480,
        XmNminWidth, 640,
        XmNminHeight, 480,
        XmNmaxWidth, 640,
        XmNmaxHeight, 480,
        NULL);
    form = XtVaCreateManagedWidget("form", xmFormWidgetClass, app_shell,
        NULL);

    /* Menu bar */
    foo[0] = XmStringCreateSimple("File");
    foo[1] = XmStringCreateSimple("Music");
    foo[2] = XmStringCreateSimple("Help");
    menu = XmVaCreateSimpleMenuBar(form, "menu",
        XmVaCASCADEBUTTON, foo[0], 'F',
        XmVaCASCADEBUTTON, foo[1], 'M',
        XmVaCASCADEBUTTON, foo[2], 'H',
        NULL);
    XtVaSetValues(menu,
        XmNleftAttachment, XmATTACH_FORM,
        XmNrightAttachment, XmATTACH_FORM,
        XmNmenuHelpWidget, XtNameToWidget(menu, "button_2"),
        NULL);
    XmStringFree(foo[2]);
    XmStringFree(foo[1]);
    XmStringFree(foo[0]);

    /* Pulldown menus */
    foo[0] = XmStringCreateSimple("Open...");
    foo[1] = XmStringCreateSimple("Save As...");
    foo[2] = XmStringCreateSimple("Exit");
    foo[3] = XmStringCreateSimple("Make MIDI File...");
    foo[4] = XmStringCreateSimple("Play Music...");
    foo[5] = XmStringCreateSimple("About...");
    foo[6] = XmStringCreateSimple("Ctrl+O");
    foo[7] = XmStringCreateSimple("Ctrl+C");
    foo[8] = XmStringCreateSimple("Ctrl+P");
    XmVaCreateSimplePulldownMenu(menu, "file", 0, FileCB,
        XmVaPUSHBUTTON, foo[0], 'O', "Ctrl <Key> o", foo[6],
        XmVaPUSHBUTTON, foo[1], 'S', NULL, NULL,
        XmVaSEPARATOR,
        XmVaPUSHBUTTON, foo[2], 'x', "Ctrl <Key> c", foo[7],
        NULL);
    XmVaCreateSimplePulldownMenu(menu, "music", 1, MusicCB,
        XmVaPUSHBUTTON, foo[3], 'M', NULL, NULL,
        XmVaPUSHBUTTON, foo[4], 'P', "Ctrl <Key> p", foo[8],
        NULL);
    XmVaCreateSimplePulldownMenu(menu, "help", 2, HelpCB,
        XmVaPUSHBUTTON, foo[5], 'A', NULL, NULL,
        NULL);
    XtManageChild(menu);
    XmStringFree(foo[8]);
    XmStringFree(foo[7]);
    XmStringFree(foo[6]);
    XmStringFree(foo[5]);
    XmStringFree(foo[4]);
    XmStringFree(foo[3]);
    XmStringFree(foo[2]);
    XmStringFree(foo[1]);
    XmStringFree(foo[0]);

    /* Seed */
    foo[0] = XmStringCreateSimple("Seed");
    Seed = XtVaCreateManagedWidget("seed", xmTextFieldWidgetClass, form,
        XmNx, 108,
        XmNy, 90,
        XmNwidth, 82,
        XmNheight, 32,
        XmNmaxLength, 5,
        NULL);
    XtVaCreateManagedWidget("seedLabel", xmLabelWidgetClass, form,
        XmNx, 64,
        XmNy, 96,
        XmNwidth, 32,
        XmNheight, 16,
        XmNlabelString, foo[0],
        NULL);
    XmTextFieldSetString(Seed, "1");
    XmStringFree(foo[0]);

    /* Parts */
    foo[0] = XmStringCreateSimple("Two Parts");
    foo[1] = XmStringCreateSimple("Three Parts");
    foo[2] = XmStringCreateSimple("Four Parts");
    foo[3] = XmStringCreateSimple("Five Parts");
    foo[4] = XmStringCreateSimple("Six Parts");
    frame1 = XtVaCreateManagedWidget("frame1", xmFrameWidgetClass, form,
        XmNx, 64,
        XmNy, 160,
        XmNwidth, 128,
        XmNheight, 148,
        NULL);
    Parts = XmVaCreateSimpleRadioBox(frame1, "parts", 0, PartsCB,
        XmVaRADIOBUTTON, foo[0], NULL, NULL, NULL,
        XmVaRADIOBUTTON, foo[1], NULL, NULL, NULL,
        XmVaRADIOBUTTON, foo[2], NULL, NULL, NULL,
        XmVaRADIOBUTTON, foo[3], NULL, NULL, NULL,
        XmVaRADIOBUTTON, foo[4], NULL, NULL, NULL,
        NULL);
    XtVaSetValues(Parts, XmNspacing, 0, NULL);
    XtManageChild(Parts);
    XmToggleButtonGadgetSetState(XtNameToWidget(Parts, "button_0"),
        True, False);
    XmStringFree(foo[4]);
    XmStringFree(foo[3]);
    XmStringFree(foo[2]);
    XmStringFree(foo[1]);
    XmStringFree(foo[0]);

    /* Beats Per Minute */
    foo[0] = XmStringCreateSimple("Beats Per Minute");
    BPM = XtVaCreateWidget("bpm", xmScaleWidgetClass, form,
        XmNx, 64,
        XmNy, 344,
        XmNwidth, 128,
        XmNheight, 64,
        XmNorientation, XmHORIZONTAL,
        XmNminimum, 120,
        XmNmaximum, 240,
        XmNshowValue, True,
        XmNtitleString, foo[0],
        NULL);
    XtAddCallback(BPM, XmNvalueChangedCallback, BPMCB, NULL);
    XtManageChild(BPM);
    XmStringFree(foo[0]);

    /* Scale */
    foo[0] = XmStringCreateSimple("Pentatonic");
    foo[1] = XmStringCreateSimple("Major");
    foo[2] = XmStringCreateSimple("Natural Minor");
    foo[3] = XmStringCreateSimple("Melodic Minor");
    foo[4] = XmStringCreateSimple("Harmonic Minor");
    foo[5] = XmStringCreateSimple("Whole Tone");
    foo[6] = XmStringCreateSimple("Octatonic");
    foo[7] = XmStringCreateSimple("Chromatic");
    foo[8] = XmStringCreateSimple("Scale");
    Scale = XtVaCreateWidget("scale", xmListWidgetClass, form,
        XmNx, 224,
        XmNy, 110,
        XmNwidth, 192,
        XmNheight, 162,
        XmNitemCount, 8,
        XmNitems, foo,
        NULL);
    XmListSelectPos(Scale, 1, False);
    XtAddCallback(Scale, XmNbrowseSelectionCallback, ScaleCB, NULL);
    XtManageChild(Scale);
    XtVaCreateManagedWidget("scaleLabel", xmLabelWidgetClass, form,
        XmNx, 224,
        XmNy, 88,
        XmNwidth, 40,
        XmNheight, 16,
        XmNlabelString, foo[8],
        NULL);
    XmStringFree(foo[8]);
    XmStringFree(foo[7]);
    XmStringFree(foo[6]);
    XmStringFree(foo[5]);
    XmStringFree(foo[4]);
    XmStringFree(foo[3]);
    XmStringFree(foo[2]);
    XmStringFree(foo[1]);
    XmStringFree(foo[0]);

    /* Phase Function */
    foo[0] = XmStringCreateSimple("Phase Function");
    frame2 = XtVaCreateManagedWidget("frame2", xmFrameWidgetClass, form,
        XmNx, 218,
        XmNy, 318,
        XmNwidth, 204,
        XmNheight, 54,
        NULL);
    draw = XtVaCreateManagedWidget("draw", xmDrawingAreaWidgetClass, frame2,
        XmNwidth, 200,
        XmNheight, 50,
        NULL);
    XtAddCallback(draw, XmNexposeCallback, ExposeCB, NULL);
    XtAddCallback(draw, XmNinputCallback, InputCB, NULL);
    XtAddEventHandler(draw, Button1MotionMask, False, MotionCB, NULL);
    XtVaCreateManagedWidget("function", xmLabelWidgetClass, form,
        XmNx, 226,
        XmNy, 378,
        XmNwidth, 100,
        XmNheight, 32,
        XmNlabelString, foo[0],
        NULL);
    XmStringFree(foo[0]);

    /* Phase Length */
    foo[0] = XmStringCreateSimple("Pattern Length");
    Length = XtVaCreateWidget("length", xmScaleWidgetClass, form,
        XmNx, 448,
        XmNy, 80,
        XmNwidth, 128,
        XmNheight, 64,
        XmNorientation, XmHORIZONTAL,
        XmNminimum, 2,
        XmNmaximum, 24,
        XmNshowValue, True,
        XmNtitleString, foo[0],
        NULL);
    XtAddCallback(Length, XmNvalueChangedCallback, LengthCB, NULL);
    XtManageChild(Length);
    XmStringFree(foo[0]);

    /* Humanize Touch */
    foo[0] = XmStringCreateSimple("Humanize Touch");
    HT = XtVaCreateWidget("ht", xmScaleWidgetClass, form,
        XmNx, 448,
        XmNy, 168,
        XmNwidth, 128,
        XmNheight, 64,
        XmNorientation, XmHORIZONTAL,
        XmNminimum, 1,
        XmNmaximum, 48,
        XmNshowValue, True,
        XmNtitleString, foo[0],
        NULL);
    XtAddCallback(HT, XmNvalueChangedCallback, HTCB, NULL);
    XtManageChild(HT);
    XmStringFree(foo[0]);

    /* Random Staccato */
    foo[0] = XmStringCreateSimple("Random Staccato");
    RS = XtVaCreateWidget("rs", xmScaleWidgetClass, form,
        XmNx, 448,
        XmNy, 256,
        XmNwidth, 128,
        XmNheight, 64,
        XmNorientation, XmHORIZONTAL,
        XmNminimum, 1,
        XmNmaximum, 48,
        XmNshowValue, True,
        XmNtitleString, foo[0],
        NULL);
    XtAddCallback(RS, XmNvalueChangedCallback, RSCB, NULL);
    XtManageChild(RS);
    XmStringFree(foo[0]);

    /* Total Duration */
    foo[0] = XmStringCreateSimple("Total Duration");
    Duration = XtVaCreateWidget("duration", xmScaleWidgetClass, form,
        XmNx, 448,
        XmNy, 344,
        XmNwidth, 128,
        XmNheight, 64,
        XmNorientation, XmHORIZONTAL,
        XmNminimum, 100,
        XmNmaximum, 800,
        XmNshowValue, True,
        XmNtitleString, foo[0],
        NULL);
    XtAddCallback(Duration, XmNvalueChangedCallback, DurationCB, NULL);
    XtManageChild(Duration);
    XmStringFree(foo[0]);
    XtRealizeWidget(app_shell);

    /* Initialize pixmap */
    display = XtDisplay(draw);
    screen = XtScreen(draw);
    d = XtWindow(draw);
    values.foreground = XAllocNamedColor(display,
        DefaultColormapOfScreen(screen), "#fafafafad2d2",
        &screen_def, &exact_def) ? screen_def.pixel :
        WhitePixelOfScreen(screen);
    values.line_width = 2;
    gc = XCreateGC(display, d, GCForeground | GCLineWidth, &values);
    pixmap = XCreatePixmap(display, d, 200, 50,
        DefaultDepthOfScreen(screen));
    resetphase();
    XtAppMainLoop(app);
    return EXIT_SUCCESS;
}
